# =====================================================================================================================
#                                                       VIIA
#                                                  Objectnr: XXXX
#                                             Structural engineer: Name
# =====================================================================================================================
# TEMPLATE NLTH v5.4
# viiapackage version v##.#.#

# Copyright VIIA

# Note: The mainscript template is divided into several sections following the steps in NLTH protocol. Functions that
# can be used in each step are placed under the corresponding section. Some functions can only be used under certain
# conditions, please check the docstring before calling them.

# =====================================================================================================================
#                                                    Initial startup
# =====================================================================================================================

# Import the viiaPackage
import warnings
from viiapackage import *

# Import your own functionality here (object specific)

# =====================================================================================================================
#                                                    Booleans
# =====================================================================================================================

# Boolean to add boundary springs during static analysis
boundary_spring_static = False

# Analysis
analysis = ['A1']
num_modes = 1000
signal = ['S1']  # Select (multiple) from S1, S2, S3, S5, S6, S7 and S9
run_analysis = False
pictures = True

# Boolean for mesh check
mesh_check = True

# Boolean for applying connections
detailing = True
model_plots = False

# Boolean for cost quantities
cost_quantities = False

# =====================================================================================================================
#                                              Initiate session
# =====================================================================================================================

# Initiate the session
project = viia_create_project(project_name='XXXX', version_nr=1)

# Load the model in the json that was created with the model script (model sub folder)
project.viia_read_dump('model/XXXX-model', False)

# =====================================================================================================================
#                                   Fixed base model imposed loads (step 7 protocol)
# =====================================================================================================================

# Converting imposed load to self weight
project.viia_convert_imposed_loads_to_self_weight()

# Create the load combination for dead load (static situation)
project.viia_create_load_combination_static()

# =====================================================================================================================
#                                      Fixed base model supports (step 8 protocol)
# =====================================================================================================================

# Add boundary springs which prevent beams and columns from spinning about their own axes
if boundary_spring_static:
    project.viia_create_all_boundary_springs_for_static_analysis(spring_stiffness=50)

# Apply pile foundation fixed base

# Set the coordinates of the piles
# Input as floats for x- and y-coordinate of the piles, dimensions in [m]
pile_coordinates = [
    [1.00000E+00, 9.54000E+00],
    [3.50000E+00, 9.54000E+00],
    [6.00000E+00, 9.54000E+00]]

if any(a in ['A1', 'A2', 'A3', 'A4'] for a in analysis):
    # Create piles, make sure to add the pile-group in MYVIIA first
    project.viia_create_piles(coordinates=pile_coordinates, pile_group='A')

# Apply shallow foundation fixed base

# You can specify additional and to be excluded surfaces
if any(a in ['A1', 'A2', 'A3', 'A4'] for a in analysis):
    project.viia_create_supports()

# =====================================================================================================================
#                                        Checking the model (step C1 protocol)
# =====================================================================================================================

# Create the model to check the mesh (required to run in DIANA)
if any(a in ['A1', 'A2', 'A3', 'A4'] for a in analysis):

    # Create the fixed base model
    project.viia_create_model()

    # Mesh check
    if mesh_check:
        project.viia_create_datfile()
        project.viia_check_mesh()

# =====================================================================================================================
#                              Fixed base linear analysis (step A1, A2, A3, A4 protocol)
# =====================================================================================================================

# Linear elastic static calculation (step A1 protocol)
if 'A1' in analysis:
    project.viia_linear_properties()
    project.viia_analysis(analysis_nr='A1', run=run_analysis)
    project.viia_non_linear_properties()

# Rob-test calculation (step A2 protocol) (Optional)
if 'A2' in analysis:
    project.viia_create_loads(load='Rob-test')
    project.viia_create_load_combination_rob_test()
    project.viia_update_pile_supports_for_rob_test()
    project.viia_analysis(analysis_nr='A2', run=run_analysis)
    project.viia_remove_temporarily_pile_supports_for_rob_test()
    project.viia_remove_loads(load_type='Rob-test')
    project.viia_remove_load_combination_rob_test()

# Eigenvalue analysis (step A3 protocol) (Optional)
if 'A3' in analysis:
    project.viia_linear_properties()
    project.viia_analysis(analysis_nr='A3', run=run_analysis, nmodes=num_modes)
    project.viia_non_linear_properties()

# Linear time history analysis (step A4 protocol) (Optional)
if 'A4' in analysis:
    project.viia_linear_properties()
    project.viia_create_loads('Base motion')
    project.viia_add_basemotion_signals()
    project.viia_create_load_combination_base_motion(signal[0])
    project.viia_analysis(analysis_nr='A4', run=run_analysis, signals=signal)
    project.viia_non_linear_properties()

# =====================================================================================================================
#                                       Flexbase model supports (step 9 protocol)
# =====================================================================================================================

# Create flexbase supports if any analysis is selected after A4
if any(a not in ['A1', 'A2', 'A3', 'A4'] for a in analysis):

    # Apply pile foundation flexbase

    # If there is pile foundation (use pile tool for sending data to MYVIIA first before creating flexbase piles)
    project.viia_create_piles(coordinates=pile_coordinates, support_type='FlexBaseGlobal', pile_group='A')

    # Apply shallow foundation flexbase

    # Select for FexBaseGlobal or FlexBaseFinal. Only select FlexBaseFinal if geotechnical advisor has provided input
    # on MYVIIA for the shallow foundations, else the values are calculated based on average values for FlexBaseGlobal
    project.viia_create_supports(support_type='FlexBaseGlobal')

# =====================================================================================================================
#                                           Detailing (step 11 protocol)
# =====================================================================================================================

# Apply connections
# Auto create functions assume proper connected shapes (performed in modelscript)
if detailing:

    # Add any manual detailing here

    # Apply all interfaces automatically based on BoD
    project.viia_auto_interfaces()

    # Apply all hinges automatically based on BoD
    project.viia_auto_hinges()

# Create pictures for building structural setup appendix including the pictures of connections
if model_plots:

    # Same steps as in the modelscript, but now it includes the connections
    project.viia_create_model_plots()

    # Create the building structural setup appendix
    project.viia_create_model_plots_appendix()

# =====================================================================================================================
#                                         Check the model - flexbase (step C2, M1 protocol)
# =====================================================================================================================

# Create the model to check the mesh (required to run in DIANA)
if any(a not in ['A1', 'A2', 'A3', 'A4'] for a in analysis) and project.version == 1:

    # Create the flexbase model
    project.viia_create_model()
    if mesh_check or pictures:
        project.viia_create_datfile()

    # Mesh check
    if mesh_check:
        project.viia_check_mesh()

    # Create model pictures for the TVA
    if pictures:
        project.viia_model_pictures(backside=False)

# =====================================================================================================================
#                                  Flexbase eigenfrequency analysis (step A7 protocol)
# =====================================================================================================================

# Eigenvalue analysis flexbase (step A7 protocol)
if 'A7' in analysis:
    project.viia_analysis(analysis_nr='A7', run=run_analysis, nmodes=num_modes)

# =====================================================================================================================
#                                Flexbase nonlinear analysis (step A10, A12 protocol)
# =====================================================================================================================

# Nonlinear static flexbase analysis (step A10)
if 'A10' in analysis:
    # Convert imposed loads to self weight (as extra density), imposed loads are created in modelscript
    project.viia_convert_self_weight_to_imposed_loads()
    project.viia_analysis(analysis_nr='A10', run=run_analysis)
    project.viia_convert_imposed_loads_to_self_weight()
    warnings.warn(
        f"Imposed Loads have been converted to self weight. If not running analysis directly (i.e. 'run_analysis' "
        f"is set to False), then analysis is incorrect. Close DIANA and open A10 from "
        f"{project.diana_settings.current_analysis_folder}.")

# Nonlinear time history flexbase analysis (step A12)
if 'A12' in analysis:
    project.viia_create_loads('Base motion')
    project.viia_add_basemotion_signals()
    project.viia_create_load_combination_base_motion(signal[0])
    project.viia_analysis(analysis_nr='A12', run=run_analysis, signals=signal)

# =====================================================================================================================
#                                      Strengthening phase (step 13, A13, A15 protocol)
# =====================================================================================================================

# Modelling of strengthening measure (TVA Phase)
if project.version > 1:

    # Apply the measures for strengthening here

    # Create the model
    project.viia_create_model()
    if mesh_check or pictures:
        project.viia_create_datfile()

    # Mesh check
    if mesh_check:
        project.viia_check_mesh()

    # Create model pictures for the TVA, run this command in DIANA after creating model and generating dat file.
    if pictures:
        project.viia_model_pictures(backside=False)

    # Nonlinear static flexbase analysis (step A13)(Optional)
    if 'A13' in analysis:

        # Convert imposed loads to self weight (as extra density), imposed loads are created in modelscript
        project.viia_convert_self_weight_to_imposed_loads()
        project.viia_analysis(analysis_nr='A13', run=run_analysis)
        project.viia_convert_imposed_loads_to_self_weight()
        warnings.warn(
            f"Imposed Loads have been converted to self weight. If not running analysis directly (i.e. 'run_analysis' "
            f"is set to False), then analysis is incorrect. Close DIANA and open A13 from "
            f"{project.diana_settings.current_analysis_folder}.")

    # Nonlinear time history flexbase analysis with strengthening measures (step A15)(Optional)
    # Result handling is done in separate script
    if 'A15' in analysis:
        project.viia_create_loads('Base motion')
        project.viia_add_basemotion_signals()
        project.viia_create_load_combination_base_motion(signal[0])
        project.viia_analysis(analysis_nr='A15', run=run_analysis, signals=signal)

# =====================================================================================================================
#                                  GMC and cost engineer (step 14, 15 protocol)
# =====================================================================================================================

# Generate output for the cost engineer
if cost_quantities:

    # Collect the facades which are in the model
    facades = project.viia_get_facades()

    # Send data to cost engineer
    project.viia_create_cost_key_figures(facade_list=facades)

# =====================================================================================================================
#                                        End of the analyses of the object
# =====================================================================================================================

# End of the mainscript
project.viia_print_execution_time()
